<?php

namespace Drupal\ajax_placeholder\Element;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Render\Element\RenderElement;

/**
 * Provides a AJAX placeholder render element.
 *
 * @code
 * $build['awesome'] = [
 *   '#type' => 'ajax_placeholder',
 *   '#callback' => ['mymodule.service:callback', [$arg1, $arg2]],
 * ];
 * @endcode
 *
 * @RenderElement("ajax_placeholder")
 */
class AjaxPlaceholder extends RenderElement {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    return [
      '#pre_render' => [
        [self::class, 'preRender'],
      ],
    ];
  }

  /**
   * Pre-render callback for AJAX placeholders.
   *
   * @param array $element
   *   The renderable array for the element.
   *
   * @return array
   *   The passed in element with changes made to attributes depending on
   *   context.
   */
  public static function preRender(array $element) {
    // Set the callback data in the tempstore and use a temporary hash to
    // retrieve it.
    $tempstore = \Drupal::service('tempstore.shared')->get('ajax_placeholder');
    $hash = md5(Json::encode($element['#callback']));
    $tempstore->set($hash, [
      'url' => \Drupal::request()->getRequestUri(),
      'callback' => $element['#callback'],
    ]);
    return [
      'container' => [
        '#type' => 'container',
        '#callback' => '',
        '#attributes' => [
          'class' => 'js-ajax-placeholder',
          'data-hash' => $hash,
        ],
        'content' => [
          '#markup' => t('Loading...'),
        ],
        '#attached' => [
          'library' => ['ajax_placeholder/ajax'],
        ],
      ],
    ];
  }

}
